#ifndef __IMMS_H
#define __IMMS_H

#include <string>
#include <fstream>
#include <memory>
#include <map>

#include "immsdb.h"
#include "songinfo.h"
#include "xidle.h"

using std::string;
using std::ofstream;
using std::auto_ptr;
using std::map;

#define ROUND(x) (int)((x) + 0.5)

int imms_random(int max);

typedef map<int, int> CandidateMap;

// IMMS, UMMS, we all MMS for XMMS?

class Imms
{
public:
    Imms();

    // Public interface
    int  select_next();
    bool add_candidate(int playlist_num, string path, bool urgent = false);

    void start_song(string path);
    void end_song(bool at_the_end, bool jumped, bool bad);

    void playlist_changed();
    void pump();

    void setup(const char* _email, bool _use_xidle);

protected:
    // Helper functions
    bool fetch_song_info(const string &path);
    void parse_song_info(const string &path);
    void print_song_info(const string &path);
    int select_next_helper();

    // State variables
    string email;
    bool last_skipped, last_jumped, use_xidle;
    int quality, timehold, next_timehold, last_handpicked;
    int max_composite, min_composite, have_candidates;
    int current_rating;

    // Helper objects
    auto_ptr<ImmsDb> immsdb;
    ofstream fout;
    SongInfo songinfo;
    CandidateMap candidates;
    XIdle xidle;

    // Song info cache
    int rating, link; 
    bool identified, unrated;
    time_t last_played;
    string artist, title;
};


#endif
